package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.method;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPartContext;
import cn.estsh.i3plus.mes.pcn.api.iservice.base.IPartService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 缓存零件通用工步方法
 * @Author : wangjie
 **/
@Slf4j
@Service("mesPartDataMapSaveStepService")
public class MesPartDataMapSaveStepService extends BaseStepService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IPartService partService;

    //保存零件数据信息
    public StepResult savePartDataMap2PPC(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, List<MesProductionPartContext> productionPartContextList, Boolean isBackError, Boolean isFilter, Boolean isSave) {

        //从一模多腔信息中搜集零件编码并去重
        List<String> partNoList = (productionPartContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getPartNo()))).map(MesProductionPartContext::getPartNo).collect(Collectors.toList())).stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());

        return savePartDataMap(reqBean, resultBean, stepResult, partNoList, isBackError, isFilter, isSave);

    }

    //保存零件数据信息
    public StepResult savePartDataMap2PPC(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, List<MesProductionPartContext> productionPartContextList, Boolean isBackError, Boolean isFilter) {

        //从一模多腔信息中搜集零件编码并去重
        List<String> partNoList = (productionPartContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getPartNo()))).map(MesProductionPartContext::getPartNo).collect(Collectors.toList())).stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());

        return savePartDataMap(reqBean, resultBean, stepResult, partNoList, isBackError, isFilter, true);

    }

    //保存零件数据信息 【isBackError=false: 抛出异常】【isFilter=false: 重新查询所有零件；true: 过滤之后查询未缓存的零件之后再进行合并前面已缓存过的零件】
    public StepResult savePartDataMap(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, List<String> partNoList, Boolean isBackError, Boolean isFilter) {

        return savePartDataMap(reqBean, resultBean, stepResult, partNoList, isBackError, isFilter, true);

    }

    //保存零件数据信息 【isBackError=false: 抛出异常】【isFilter=false: 重新查询所有零件；true: 过滤之后查询未缓存的零件之后再进行合并前面已缓存过的零件】
    public StepResult savePartDataMap(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, List<String> partNoList, Boolean isBackError, Boolean isFilter, Boolean isSave) {

        if (CollectionUtils.isEmpty(partNoList)) return stepResult;

        //从上下文中取出零件信息集合
        Map<String, MesPart> partDataMap = productionProcessContextStepService.getPartDataContext(reqBean);

        //验证上下文中取出零件信息集合是否匹配当前搜集的零件编码
        if (!CollectionUtils.isEmpty(partDataMap) && partDataMap.keySet().containsAll(partNoList)) return stepResult.obj(partDataMap);

        //新零件MAP
        Map<String, MesPart> finalPartDataMap;
        if (!isFilter) {
            //根据物料编码集合查询物料信息
            finalPartDataMap = partService.getPartMap(reqBean.getOrganizeCode(), partNoList);
        } else {
            //已缓存的零件号
            List<String> containPartNoList = CollectionUtils.isEmpty(partDataMap) ? null : partNoList.stream().filter(o -> (!StringUtils.isEmpty(o) && partDataMap.containsKey(o))).collect(Collectors.toList());
            //未缓存的零件号
            List<String> unContainPartNoList = CollectionUtils.isEmpty(partDataMap) ? partNoList : partNoList.stream().filter(o -> (!StringUtils.isEmpty(o) && !partDataMap.containsKey(o))).collect(Collectors.toList());
            //根据未缓存的零件号集合查询物料信息
            finalPartDataMap = partService.getPartMap(reqBean.getOrganizeCode(), unContainPartNoList);
            //合并零件号
            if (!CollectionUtils.isEmpty(finalPartDataMap) && !CollectionUtils.isEmpty(containPartNoList)) containPartNoList.stream().filter(o -> !StringUtils.isEmpty(o)).forEach(o -> finalPartDataMap.put(o, partDataMap.get(o)));
        }

        //零件信息缺失 抛出异常
        List<String> filterList = CollectionUtils.isEmpty(finalPartDataMap) ? null : partNoList.stream().filter(o -> (!StringUtils.isEmpty(o) && !finalPartDataMap.containsKey(o))).collect(Collectors.toList());

        //保存上下文零件数据信息
        if (!CollectionUtils.isEmpty(finalPartDataMap) && CollectionUtils.isEmpty(filterList)) {
            if (isSave) productionProcessContextStepService.dispatchPartDataContext(reqBean, finalPartDataMap);
            return stepResult.obj(finalPartDataMap);
        }

        //验证错误 根据isBackError判断是否抛出异常
        stepResult.isCompleted(false).msg(String.format("请检查零件信息,零件编码%s信息不存在!", CollectionUtils.isEmpty(finalPartDataMap) ? partNoList.toString() : filterList.toString()));

        if (!isBackError) stepExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), stepResult.getMsg());

        return stepResult;

    }

}
